@@ -65,7 +65,7 @@
 **************************************/
 int Lizard_versionNumber (void) { return LIZARD_VERSION_NUMBER; }
 int Lizard_compressBound(int isize)  { return LIZARD_COMPRESSBOUND(isize); }
-int Lizard_sizeofState_MinLevel() { return Lizard_sizeofState(LIZARD_MIN_CLEVEL); }
+int Lizard_sizeofState_MinLevel(void) { return Lizard_sizeofState(LIZARD_MIN_CLEVEL); }
 
 
 
@@ -392,6 +392,7 @@ Lizard_stream_t* Lizard_initStream(Lizard_stream_t* ctx, int compressionLevel)
 Lizard_stream_t* Lizard_createStream(int compressionLevel) 
 { 
     Lizard_stream_t* ctx = Lizard_initStream(NULL, compressionLevel);
+    if (ctx) ctx->base = NULL;
     return ctx; 
 }
 
@@ -429,7 +430,7 @@ int Lizard_loadDict(Lizard_stream_t* Lizard_streamPtr, const char* dictionary, i
         dictionary += dictSize - LIZARD_DICT_SIZE;
         dictSize = LIZARD_DICT_SIZE;
     }
-    Lizard_init (ctxPtr, (const BYTE*)dictionary);
+    Lizard_init(ctxPtr, (const BYTE*)dictionary);
     if (dictSize >= HASH_UPDATE_LIMIT) Lizard_Insert (ctxPtr, (const BYTE*)dictionary + (dictSize - (HASH_UPDATE_LIMIT-1)));
     ctxPtr->end = (const BYTE*)dictionary + dictSize;
     return dictSize;
@@ -550,8 +551,8 @@ int Lizard_compress_continue (Lizard_stream_t* ctxPtr,
                                             const char* source, char* dest,
                                             int inputSize, int maxOutputSize)
 {
-    /* auto-init if forgotten */
-    if (ctxPtr->base == NULL) Lizard_init (ctxPtr, (const BYTE*) source);
+    /* auto-init if required */
+    if (ctxPtr->base == NULL) Lizard_init(ctxPtr, (const BYTE*) source);
 
     /* Check overflow */
     if ((size_t)(ctxPtr->end - ctxPtr->base) > 2 GB) {
@@ -586,7 +587,7 @@ int Lizard_compress_extState (void* state, const char* src, char* dst, int srcSi
 
     /* initialize stream */
     Lizard_initStream(ctx, compressionLevel);
-    Lizard_init ((Lizard_stream_t*)state, (const BYTE*)src);
+    Lizard_init((Lizard_stream_t*)state, (const BYTE*)src);
 
     return Lizard_compress_generic (state, src, dst, srcSize, maxDstSize);
 }
