@@ -1,6 +1,7 @@
 /*
     bench.h - Demo program to benchmark open-source compression algorithm
     Copyright (C) Yann Collet 2012-2016
+    Copyright (C) Przemyslaw Skibinski 2016-2017
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
     You can contact the author at :
-    - LZ5 source repository : https://github.com/inikep/lz5
+    - Lizard source repository : https://github.com/inikep/lizard
 */
 #ifndef BENCH_H_12
 #define BENCH_H_12
